/*!
\file    djgraph.h
\brief   Graphics functionality
\author  David Joffe
*/
/*--------------------------------------------------------------------------*/
#ifndef _DJGRAPH_H_
#define _DJGRAPH_H_

#ifdef USESDL
#include "SDL.h"
#else

#ifdef WIN32
#include <windows.h>
#include <ddraw.h>
#endif

// If Windows, include DirectDraw stuff
#ifdef WIN32
extern HINSTANCE g_hInstance;
extern HWND      g_hWnd;
extern void      djgInitWin32( int hInstance );
#else
// Linux, include GGI stuff
//#include <ggi/ggi.h>
#endif
#endif

#include "djimage.h"
#include "djtypes.h"
/*--------------------------------------------------------------------------*/
#define PIXEL32TO16(i) ( ((((i&0xFF0000)>>16) / 8) << 11) |   ((((i&  0xFF00)>> 8) / 4) <<  5) |   ((((i&    0xFF)    ) / 8)      ) )

class djVisual
{
public:
	djVisual()
	{
#ifdef USESDL
		pSurface = NULL;
		colorfore = djColor(255,255,255);
		colorback = djColor(0,0,0);
#else
#ifdef WIN32
		vis = NULL;
		m_bPrimary = false;
		colorfore = djColor(255,255,255);
		colorback = djColor(0,0,0);
#else
#endif
		m_pBuffer = NULL;
#endif
		width    = 0;
		height   = 0;
		stride   = 0;
		bpp      = 0;
		pixwidth = 0;
		m_bFullscreen = false;
	}
#ifdef USESDL
	SDL_Surface *pSurface;
	djColor                  colorfore;
	djColor                  colorback;
#else
#ifdef WIN32
	LPDIRECTDRAWSURFACE      vis;
	char *                   m_pBuffer;
	bool                     m_bPrimary; // Must know for primitive offsets
	djColor                  colorfore;
	djColor                  colorback;
#else
	ggi_visual_t             vis;
	const ggi_directbuffer * m_pBuffer;
#endif
#endif
	int                      width;
	int                      height;
	int                      stride;
	int                      bpp;         // 32 ... 24? (pixsize) pixdepth=24, pixsize=32 width=4
	int                      pixwidth;    // 4
	bool                     m_bFullscreen;
};
/*--------------------------------------------------------------------------*/
// Housekeeping

//! Initialize the library
extern int        djgInit();
//! Library cleanup
extern int        djgDone();
/*--------------------------------------------------------------------------*/
// Color management

extern unsigned int djgMapColor( djVisual *pVis, djColor color );
/*--------------------------------------------------------------------------*/
// Visual management

//! Create a visual
extern djVisual *djgOpenVisual( const char * vistype, int w, int h, int bpp = 0, bool bBackbuffer=false);
//! Destroy a visual
extern void      djgCloseVisual( djVisual * pVis );
//! Lock a visual for surface buffer drawing
extern void      djgLock( djVisual * pVis );
//! Unlock a visual
extern void      djgUnlock( djVisual * pVis );
//! Flush any pending drawing operations to visual buffer
extern void      djgFlush( djVisual * pVis );
/*--------------------------------------------------------------------------*/
// 2D drawing functions

//-- Color functions

//! set foreground and background colors
extern void       djgSetColor( djVisual *pVis, djColor clrFore, djColor clrBack );
//! set foreground color
extern void       djgSetColorFore( djVisual * pVis, djColor color );
//! set background color
extern void       djgSetColorBack( djVisual * pVis, djColor color );

//-- Drawing primitives

//! clear a visual
extern void       djgClear( djVisual * pVis );
//! Put pixel
extern void       djgPutPixel( djVisual * pVis, int x, int y, int r, int g, int b );
//! Put pixel
extern void       djgPutPixel( djVisual * pVis, int x, int y, djColor color );
//! draw rectangle in current color
extern void       djgDrawRectangle( djVisual * pVis, int x, int y, int w, int h );
//! draw filled box in current color
extern void       djgDrawBox( djVisual * pVis, int x, int y, int w, int h );
//! draw horizontal line in current color
extern void       djgDrawHLine( djVisual * pVis, int x, int y, int n );
//! draw vertical line in current color
extern void       djgDrawVLine( djVisual * pVis, int x, int y, int n );

//-- Image blitting

//! Draw (w,h) pixels of an image at (x,y)
extern void       djgDrawImage( djVisual *pVis, djImage *pImage, int x, int y, int w, int h );
//! Draw (w,h) pixels of an image at (xD,yD) from (xS,yS) on image
extern void       djgDrawImage( djVisual *pVis, djImage *pImage, int xS, int yS, int xD, int yD, int w, int h );
//! Draw (w,h) pixels of an image at (xD,yD) from (xS,yS) on image with transparency
extern void       djgDrawImageAlpha( djVisual *pVis, djImage *pImage, int xS, int yS, int xD, int yD, int w, int h );
//! Draw (w,h) pixels from a visual at (xD,yD) from (xS,yS)
extern void       djgDrawVisual( djVisual *pDest, djVisual *pSrc, int xD, int yD, int xS, int yS, int w, int h );
//! flip one visual to another
extern void       djgFlip( djVisual * pVisDest, djVisual * pVisSrc );


#endif

